@@ -209,13 +209,6 @@ svn_error_t *svn_swig_pl_thunk_ssl_client_cert_pw_prompt
    svn_boolean_t may_save,
    apr_pool_t *pool);
 
-/* thunked callback for svn_ra_get_wc_prop_func_t */
-svn_error_t *thunk_get_wc_prop(void *baton,
-                               const char *relpath,
-                               const char *name,
-                               const svn_string_t **value,
-                               apr_pool_t *pool);
-
 /* Thunked version of svn_wc_notify_func_t callback type */
 void svn_swig_pl_notify_func(void * baton,
                              const char *path,
@@ -278,11 +271,6 @@ void svn_swig_pl_make_editor(svn_delta_editor_t **editor,
                              SV *perl_editor,
                              apr_pool_t *pool);
 
-void svn_swig_pl_wrap_window_handler(svn_txdelta_window_handler_t *handler,
-                                     void **h_baton,
-                                     SV *callback,
-                                     apr_pool_t *pool);
-
 /* svn_stream_t helpers */
 svn_error_t *svn_swig_pl_make_stream(svn_stream_t **stream, SV *obj);
 SV *svn_swig_pl_from_stream(svn_stream_t *stream);
