@@ -1,30 +1,37 @@
 /* txn-table.c : operations on the `transactions' table
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ *    Licensed to the Apache Software Foundation (ASF) under one
+ *    or more contributor license agreements.  See the NOTICE file
+ *    distributed with this work for additional information
+ *    regarding copyright ownership.  The ASF licenses this file
+ *    to you under the Apache License, Version 2.0 (the
+ *    "License"); you may not use this file except in compliance
+ *    with the License.  You may obtain a copy of the License at
  *
- * This software is licensed as described in the file COPYING, which
- * you should have received as part of this distribution.  The terms
- * are also available at http://subversion.tigris.org/license-1.html.
- * If newer versions of this license are posted there, you may use a
- * newer version instead, at your option.
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
- * This software consists of voluntary contributions made by many
- * individuals.  For exact contribution history, see the revision
- * history and logs, available at http://subversion.tigris.org/.
+ *    Unless required by applicable law or agreed to in writing,
+ *    software distributed under the License is distributed on an
+ *    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *    KIND, either express or implied.  See the License for the
+ *    specific language governing permissions and limitations
+ *    under the License.
  * ====================================================================
  */
 
 #include <string.h>
 #include <assert.h>
+
 #include "bdb_compat.h"
 
 #include "svn_pools.h"
+#include "private/svn_skel.h"
+
 #include "dbt.h"
 #include "../err.h"
 #include "../fs.h"
 #include "../key-gen.h"
-#include "../util/skel.h"
 #include "../util/fs_skels.h"
 #include "../trail.h"
 #include "../../libsvn_fs/fs-loader.h"
@@ -37,7 +44,7 @@
 static svn_boolean_t
 is_committed(transaction_t *txn)
 {
-  return (txn->kind == transaction_kind_committed) ? TRUE : FALSE;
+  return (txn->kind == transaction_kind_committed);
 }
 
 
@@ -51,11 +58,11 @@ svn_fs_bdb__open_transactions_table(DB **transactions_p,
 
   BDB_ERR(svn_fs_bdb__check_version());
   BDB_ERR(db_create(&txns, env, 0));
-  BDB_ERR(txns->open(SVN_BDB_OPEN_PARAMS(txns, NULL),
-                     "transactions", 0, DB_BTREE,
-                     open_flags, 0666));
+  BDB_ERR((txns->open)(SVN_BDB_OPEN_PARAMS(txns, NULL),
+                       "transactions", 0, DB_BTREE,
+                       open_flags, 0666));
 
-  /* Create the `next-id' table entry.  */
+  /* Create the `next-key' table entry.  */
   if (create)
   {
     DBT key, value;
@@ -78,7 +85,7 @@ svn_fs_bdb__put_txn(svn_fs_t *fs,
                     apr_pool_t *pool)
 {
   base_fs_data_t *bfd = fs->fsap_data;
-  skel_t *txn_skel;
+  svn_skel_t *txn_skel;
   DBT key, value;
 
   /* Convert native type to skel. */
@@ -89,11 +96,9 @@ svn_fs_bdb__put_txn(svn_fs_t *fs,
   svn_fs_base__str_to_dbt(&key, txn_name);
   svn_fs_base__skel_to_dbt(&value, txn_skel, pool);
   svn_fs_base__trail_debug(trail, "transactions", "put");
-  SVN_ERR(BDB_WRAP(fs, _("storing transaction record"),
-                   bfd->transactions->put(bfd->transactions, trail->db_txn,
-                                          &key, &value, 0)));
-
-  return SVN_NO_ERROR;
+  return BDB_WRAP(fs, N_("storing transaction record"),
+                  bfd->transactions->put(bfd->transactions, trail->db_txn,
+                                         &key, &value, 0));
 }
 
 
@@ -115,7 +120,7 @@ allocate_txn_id(const char **id_p,
 
   /* Get the current value associated with the `next-key' key in the table.  */
   svn_fs_base__trail_debug(trail, "transactions", "get");
-  SVN_ERR(BDB_WRAP(fs, "allocating new transaction ID (getting 'next-key')",
+  SVN_ERR(BDB_WRAP(fs, N_("allocating new transaction ID (getting 'next-key')"),
                    bfd->transactions->get(bfd->transactions, trail->db_txn,
                                           &query,
                                           svn_fs_base__result_dbt(&result),
@@ -134,8 +139,7 @@ allocate_txn_id(const char **id_p,
   db_err = bfd->transactions->put(bfd->transactions, trail->db_txn,
                                   &query, &result, 0);
 
-  SVN_ERR(BDB_WRAP(fs, "bumping next transaction key", db_err));
-  return SVN_NO_ERROR;
+  return BDB_WRAP(fs, N_("bumping next transaction key"), db_err);
 }
 
 
@@ -181,11 +185,9 @@ svn_fs_bdb__delete_txn(svn_fs_t *fs,
   /* Delete the transaction from the `transactions' table. */
   svn_fs_base__str_to_dbt(&key, txn_name);
   svn_fs_base__trail_debug(trail, "transactions", "del");
-  SVN_ERR(BDB_WRAP(fs, "deleting entry from 'transactions' table",
-                   bfd->transactions->del(bfd->transactions,
-                                          trail->db_txn, &key, 0)));
-
-  return SVN_NO_ERROR;
+  return BDB_WRAP(fs, N_("deleting entry from 'transactions' table"),
+                  bfd->transactions->del(bfd->transactions,
+                                         trail->db_txn, &key, 0));
 }
 
 
@@ -199,7 +201,7 @@ svn_fs_bdb__get_txn(transaction_t **txn_p,
   base_fs_data_t *bfd = fs->fsap_data;
   DBT key, value;
   int db_err;
-  skel_t *skel;
+  svn_skel_t *skel;
   transaction_t *transaction;
 
   /* Only in the context of this function do we know that the DB call
@@ -213,10 +215,10 @@ svn_fs_bdb__get_txn(transaction_t **txn_p,
 
   if (db_err == DB_NOTFOUND)
     return svn_fs_base__err_no_such_txn(fs, txn_name);
-  SVN_ERR(BDB_WRAP(fs, "reading transaction", db_err));
+  SVN_ERR(BDB_WRAP(fs, N_("reading transaction"), db_err));
 
   /* Parse TRANSACTION skel */
-  skel = svn_fs_base__parse_skel(value.data, value.size, pool);
+  skel = svn_skel__parse(value.data, value.size, pool);
   if (! skel)
     return svn_fs_base__err_corrupt_txn(fs, txn_name);
 
@@ -234,7 +236,7 @@ svn_fs_bdb__get_txn_list(apr_array_header_t **names_p,
                          apr_pool_t *pool)
 {
   base_fs_data_t *bfd = fs->fsap_data;
-  apr_size_t const next_id_key_len = strlen(NEXT_KEY_KEY);
+  apr_size_t const next_key_key_len = strlen(NEXT_KEY_KEY);
   apr_pool_t *subpool = svn_pool_create(pool);
   apr_array_header_t *names;
   DBC *cursor;
@@ -246,23 +248,23 @@ svn_fs_bdb__get_txn_list(apr_array_header_t **names_p,
 
   /* Create a database cursor to list the transaction names. */
   svn_fs_base__trail_debug(trail, "transactions", "cursor");
-  SVN_ERR(BDB_WRAP(fs, "reading transaction list (opening cursor)",
+  SVN_ERR(BDB_WRAP(fs, N_("reading transaction list (opening cursor)"),
                    bfd->transactions->cursor(bfd->transactions,
                                              trail->db_txn, &cursor, 0)));
 
   /* Build a null-terminated array of keys in the transactions table. */
-  for (db_err = cursor->c_get(cursor,
-                              svn_fs_base__result_dbt(&key),
-                              svn_fs_base__result_dbt(&value),
-                              DB_FIRST);
+  for (db_err = svn_bdb_dbc_get(cursor,
+                                svn_fs_base__result_dbt(&key),
+                                svn_fs_base__result_dbt(&value),
+                                DB_FIRST);
        db_err == 0;
-       db_err = cursor->c_get(cursor,
-                              svn_fs_base__result_dbt(&key),
-                              svn_fs_base__result_dbt(&value),
-                              DB_NEXT))
+       db_err = svn_bdb_dbc_get(cursor,
+                                svn_fs_base__result_dbt(&key),
+                                svn_fs_base__result_dbt(&value),
+                                DB_NEXT))
     {
       transaction_t *txn;
-      skel_t *txn_skel;
+      svn_skel_t *txn_skel;
       svn_error_t *err;
 
       /* Clear the per-iteration subpool */
@@ -274,16 +276,16 @@ svn_fs_bdb__get_txn_list(apr_array_header_t **names_p,
       svn_fs_base__track_dbt(&key, subpool);
       svn_fs_base__track_dbt(&value, subpool);
 
-      /* Ignore the "next-id" key. */
-      if (key.size == next_id_key_len
-          && 0 == memcmp(key.data, NEXT_KEY_KEY, next_id_key_len))
+      /* Ignore the "next-key" key. */
+      if (key.size == next_key_key_len
+          && 0 == memcmp(key.data, NEXT_KEY_KEY, next_key_key_len))
         continue;
 
       /* Parse TRANSACTION skel */
-      txn_skel = svn_fs_base__parse_skel(value.data, value.size, subpool);
+      txn_skel = svn_skel__parse(value.data, value.size, subpool);
       if (! txn_skel)
         {
-          cursor->c_close(cursor);
+          svn_bdb_dbc_close(cursor);
           return svn_fs_base__err_corrupt_txn
             (fs, apr_pstrmemdup(pool, key.data, key.size));
         }
@@ -292,8 +294,8 @@ svn_fs_bdb__get_txn_list(apr_array_header_t **names_p,
       if ((err = svn_fs_base__parse_transaction_skel(&txn, txn_skel,
                                                      subpool)))
         {
-          cursor->c_close(cursor);
-          return err;
+          svn_bdb_dbc_close(cursor);
+          return svn_error_trace(err);
         }
 
       /* If this is an immutable "committed" transaction, ignore it. */
@@ -301,18 +303,18 @@ svn_fs_bdb__get_txn_list(apr_array_header_t **names_p,
         continue;
 
       /* Add the transaction name to the NAMES array, duping it into POOL. */
-      (*((const char **) apr_array_push(names))) 
-        = apr_pstrmemdup(pool, key.data, key.size);
+      APR_ARRAY_PUSH(names, const char *) = apr_pstrmemdup(pool, key.data,
+                                                           key.size);
     }
 
   /* Check for errors, but close the cursor first. */
-  db_c_err = cursor->c_close(cursor);
+  db_c_err = svn_bdb_dbc_close(cursor);
   if (db_err != DB_NOTFOUND)
     {
-      SVN_ERR(BDB_WRAP(fs, "reading transaction list (listing keys)",
+      SVN_ERR(BDB_WRAP(fs, N_("reading transaction list (listing keys)"),
                        db_err));
     }
-  SVN_ERR(BDB_WRAP(fs, "reading transaction list (closing cursor)",
+  SVN_ERR(BDB_WRAP(fs, N_("reading transaction list (closing cursor)"),
                    db_c_err));
 
   /* Destroy the per-iteration subpool */
