@@ -2,17 +2,22 @@
  * ra_local.h : shared internal declarations for ra_local module
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ *    Licensed to the Apache Software Foundation (ASF) under one
+ *    or more contributor license agreements.  See the NOTICE file
+ *    distributed with this work for additional information
+ *    regarding copyright ownership.  The ASF licenses this file
+ *    to you under the Apache License, Version 2.0 (the
+ *    "License"); you may not use this file except in compliance
+ *    with the License.  You may obtain a copy of the License at
  *
- * This software is licensed as described in the file COPYING, which
- * you should have received as part of this distribution.  The terms
- * are also available at http://subversion.tigris.org/license-1.html.
- * If newer versions of this license are posted there, you may use a
- * newer version instead, at your option.
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
- * This software consists of voluntary contributions made by many
- * individuals.  For exact contribution history, see the revision
- * history and logs, available at http://subversion.tigris.org/.
+ *    Unless required by applicable law or agreed to in writing,
+ *    software distributed under the License is distributed on an
+ *    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *    KIND, either express or implied.  See the License for the
+ *    specific language governing permissions and limitations
+ *    under the License.
  * ====================================================================
  */
 
@@ -42,7 +47,7 @@ typedef struct svn_ra_local__session_baton_t
 
   /* The URL of the session, split into two components. */
   const char *repos_url;
-  svn_stringbuf_t *fs_path;  /* URI-decoded. */
+  svn_stringbuf_t *fs_path;  /* URI-decoded, always with a leading slash. */
 
   /* A repository object. */
   svn_repos_t *repos;
@@ -57,6 +62,11 @@ typedef struct svn_ra_local__session_baton_t
   /* Callbacks/baton passed to svn_ra_open. */
   const svn_ra_callbacks2_t *callbacks;
   void *callback_baton;
+
+  /* Slave auth baton */
+  svn_auth_baton_t *auth_baton;
+
+  const char *useragent;
 } svn_ra_local__session_baton_t;
 
 
@@ -64,15 +74,17 @@ typedef struct svn_ra_local__session_baton_t
 
 /** Private routines **/
 
-    
+
 
 
 /* Given a `file://' URL, figure out which portion specifies a
-   repository on local disk, and return in REPOS_URL; URI-decode and
-   return the remainder (the path *within* the repository's
-   filesystem) in FS_PATH.  Allocate the return values in POOL.
+   repository on local disk, and return that in REPOS_URL (if not
+   NULL); URI-decode and return the remainder (the path *within* the
+   repository's filesystem) in FS_PATH.  Open REPOS to the repository
+   root (if not NULL).  Allocate the return values in POOL.
    Currently, we are not expecting to handle `file://hostname/'-type
-   URLs; hostname, in this case, is expected to be the empty string. */
+   URLs; hostname, in this case, is expected to be the empty string or
+   "localhost". */
 svn_error_t *
 svn_ra_local__split_URL(svn_repos_t **repos,
                         const char **repos_url,
