@@ -4,31 +4,44 @@
 #                      (adds, deletes, reversion)
 #
 #  Subversion is a tool for revision control.
-#  See http://subversion.tigris.org for more information.
+#  See http://subversion.apache.org for more information.
 #
 # ====================================================================
-# Copyright (c) 2000-2004, 2008 CollabNet.  All rights reserved.
+#    Licensed to the Apache Software Foundation (ASF) under one
+#    or more contributor license agreements.  See the NOTICE file
+#    distributed with this work for additional information
+#    regarding copyright ownership.  The ASF licenses this file
+#    to you under the Apache License, Version 2.0 (the
+#    "License"); you may not use this file except in compliance
+#    with the License.  You may obtain a copy of the License at
 #
-# This software is licensed as described in the file COPYING, which
-# you should have received as part of this distribution.  The terms
-# are also available at http://subversion.tigris.org/license-1.html.
-# If newer versions of this license are posted there, you may use a
-# newer version instead, at your option.
+#      http://www.apache.org/licenses/LICENSE-2.0
 #
+#    Unless required by applicable law or agreed to in writing,
+#    software distributed under the License is distributed on an
+#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+#    KIND, either express or implied.  See the License for the
+#    specific language governing permissions and limitations
+#    under the License.
 ######################################################################
 
 # General modules
-import os
+import os, logging, stat
+
+logger = logging.getLogger()
 
 # Our testing module
 import svntest
 
 # (abbreviation)
-Skip = svntest.testcase.Skip
-SkipUnless = svntest.testcase.SkipUnless
-XFail = svntest.testcase.XFail
+Skip = svntest.testcase.Skip_deco
+SkipUnless = svntest.testcase.SkipUnless_deco
+XFail = svntest.testcase.XFail_deco
+Issues = svntest.testcase.Issues_deco
+Issue = svntest.testcase.Issue_deco
+Wimp = svntest.testcase.Wimp_deco
 Item = svntest.wc.StateItem
-
+exp_noop_up_out = svntest.actions.expected_noop_update_output
 
 ######################################################################
 # Tests
@@ -54,15 +67,15 @@ def add_files(sbox):
   wc_dir = sbox.wc_dir
 
   # Create some files, then schedule them for addition
-  delta_path = os.path.join(wc_dir, 'delta')
-  zeta_path = os.path.join(wc_dir, 'A', 'B', 'zeta')
-  epsilon_path = os.path.join(wc_dir, 'A', 'D', 'G', 'epsilon')
+  delta_path = sbox.ospath('delta')
+  zeta_path = sbox.ospath('A/B/zeta')
+  epsilon_path = sbox.ospath('A/D/G/epsilon')
 
   svntest.main.file_append(delta_path, "This is the file 'delta'.")
   svntest.main.file_append(zeta_path, "This is the file 'zeta'.")
   svntest.main.file_append(epsilon_path, "This is the file 'epsilon'.")
 
-  svntest.main.run_svn(None, 'add', delta_path, zeta_path, epsilon_path)
+  sbox.simple_add('delta', 'A/B/zeta', 'A/D/G/epsilon')
 
   # Make sure the adds show up as such in status
   expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
@@ -83,15 +96,15 @@ def add_directories(sbox):
   wc_dir = sbox.wc_dir
 
   # Create some directories, then schedule them for addition
-  X_path = os.path.join(wc_dir, 'X')
-  Y_path = os.path.join(wc_dir, 'A', 'C', 'Y')
-  Z_path = os.path.join(wc_dir, 'A', 'D', 'H', 'Z')
+  X_path = sbox.ospath('X')
+  Y_path = sbox.ospath('A/C/Y')
+  Z_path = sbox.ospath('A/D/H/Z')
 
   os.mkdir(X_path)
   os.mkdir(Y_path)
   os.mkdir(Z_path)
 
-  svntest.main.run_svn(None, 'add', X_path, Y_path, Z_path)
+  sbox.simple_add('X', 'A/C/Y', 'A/D/H/Z')
 
   # Make sure the adds show up as such in status
   expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
@@ -112,9 +125,9 @@ def nested_adds(sbox):
   wc_dir = sbox.wc_dir
 
   # Create some directories then schedule them for addition
-  X_path = os.path.join(wc_dir, 'X')
-  Y_path = os.path.join(wc_dir, 'A', 'C', 'Y')
-  Z_path = os.path.join(wc_dir, 'A', 'D', 'H', 'Z')
+  X_path = sbox.ospath('X')
+  Y_path = sbox.ospath('A/C/Y')
+  Z_path = sbox.ospath('A/D/H/Z')
 
   os.mkdir(X_path)
   os.mkdir(Y_path)
@@ -122,18 +135,18 @@ def nested_adds(sbox):
 
   # Now, create some files and directories to put into our newly added
   # directories
-  P_path = os.path.join(X_path, 'P')
-  Q_path = os.path.join(Y_path, 'Q')
-  R_path = os.path.join(Z_path, 'R')
+  P_path = sbox.ospath('X/P')
+  Q_path = sbox.ospath('A/C/Y/Q')
+  R_path = sbox.ospath('A/D/H/Z/R')
 
   os.mkdir(P_path)
   os.mkdir(Q_path)
   os.mkdir(R_path)
 
-  delta_path = os.path.join(X_path, 'delta')
-  epsilon_path = os.path.join(Y_path, 'epsilon')
-  upsilon_path = os.path.join(Y_path, 'upsilon')
-  zeta_path = os.path.join(Z_path, 'zeta')
+  delta_path = sbox.ospath('X/delta')
+  epsilon_path = sbox.ospath('A/C/Y/epsilon')
+  upsilon_path = sbox.ospath('A/C/Y/upsilon')
+  zeta_path = sbox.ospath('A/D/H/Z/zeta')
 
   svntest.main.file_append(delta_path, "This is the file 'delta'.")
   svntest.main.file_append(epsilon_path, "This is the file 'epsilon'.")
@@ -141,7 +154,7 @@ def nested_adds(sbox):
   svntest.main.file_append(zeta_path, "This is the file 'zeta'.")
 
   # Finally, let's try adding our new files and directories
-  svntest.main.run_svn(None, 'add', X_path, Y_path, Z_path)
+  sbox.simple_add('X', 'A/C/Y', 'A/D/H/Z')
 
   # Make sure the adds show up as such in status
   expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
@@ -168,24 +181,28 @@ def add_executable(sbox):
   sbox.build(read_only = True)
 
   def runTest(wc_dir, fileName, perm, executable):
-    fileName = os.path.join(wc_dir, fileName)
+    file_ospath = sbox.ospath(fileName)
     if executable:
       expected_out = ["*\n"]
+      expected_err = []
     else:
       expected_out = []
-    f = open(fileName,"w")
-    f.close()
-    os.chmod(fileName,perm)
-    svntest.main.run_svn(None, 'add', fileName)
-    svntest.actions.run_and_verify_svn(None, expected_out, [],
-                                       'propget', "svn:executable", fileName)
+      expected_err = '.*W200017: Property.*not found'
+
+    # create an empty file
+    open(file_ospath, "w")
+
+    os.chmod(file_ospath, perm)
+    sbox.simple_add(fileName)
+    svntest.actions.run_and_verify_svn(expected_out, expected_err,
+                                       'propget', "svn:executable", file_ospath)
 
   test_cases = [
-    ("all_exe",   0777, 1),
-    ("none_exe",  0666, 0),
-    ("user_exe",  0766, 1),
-    ("group_exe", 0676, 0),
-    ("other_exe", 0667, 0),
+    ("all_exe",   svntest.main.S_ALL_RWX, 1),
+    ("none_exe",  svntest.main.S_ALL_RW, 0),
+    ("user_exe",  svntest.main.S_ALL_RW | stat.S_IXUSR, 1),
+    ("group_exe", svntest.main.S_ALL_RW | stat.S_IXGRP, 0),
+    ("other_exe", svntest.main.S_ALL_RW | stat.S_IXOTH, 0),
     ]
   for test_case in test_cases:
     runTest(sbox.wc_dir, *test_case)
@@ -199,12 +216,7 @@ def delete_files(sbox):
   wc_dir = sbox.wc_dir
 
   # Schedule some files for deletion
-  iota_path = os.path.join(wc_dir, 'iota')
-  mu_path = os.path.join(wc_dir, 'A', 'mu')
-  rho_path = os.path.join(wc_dir, 'A', 'D', 'G', 'rho')
-  omega_path = os.path.join(wc_dir, 'A', 'D', 'H', 'omega')
-
-  svntest.main.run_svn(None, 'del', iota_path, mu_path, rho_path, omega_path)
+  sbox.simple_rm('iota', 'A/mu', 'A/D/G/rho', 'A/D/H/omega')
 
   # Make sure the deletes show up as such in status
   expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
@@ -222,17 +234,7 @@ def delete_dirs(sbox):
   wc_dir = sbox.wc_dir
 
   # Schedule some directories for deletion (this is recursive!)
-  E_path = os.path.join(wc_dir, 'A', 'B', 'E')
-  F_path = os.path.join(wc_dir, 'A', 'B', 'F')
-  H_path = os.path.join(wc_dir, 'A', 'D', 'H')
-  alpha_path = os.path.join(E_path, 'alpha')
-  beta_path  = os.path.join(E_path, 'beta')
-  chi_path   = os.path.join(H_path, 'chi')
-  omega_path = os.path.join(H_path, 'omega')
-  psi_path   = os.path.join(H_path, 'psi')
-
-  # Now, delete (recursively) the directories.
-  svntest.main.run_svn(None, 'del', E_path, F_path, H_path)
+  sbox.simple_rm('A/B/E', 'A/B/F', 'A/D/H')
 
   # Make sure the deletes show up as such in status
   expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
@@ -258,8 +260,8 @@ def check_reversion(files, output):
   output.sort()
   expected_output.sort()
   if output != expected_output:
-    print("Expected output: %s" % expected_output)
-    print("Actual output:   %s" % output)
+    logger.warn("Expected output: %s", expected_output)
+    logger.warn("Actual output:   %s", output)
     raise svntest.Failure
 
 #----------------------------------------------------------------------
@@ -271,12 +273,12 @@ def revert_add_files(sbox):
   wc_dir = sbox.wc_dir
 
   # Revert our changes recursively from wc_dir.
-  delta_path = os.path.join(wc_dir, 'delta')
-  zeta_path = os.path.join(wc_dir, 'A', 'B', 'zeta')
-  epsilon_path = os.path.join(wc_dir, 'A', 'D', 'G', 'epsilon')
+  delta_path = sbox.ospath('delta')
+  zeta_path = sbox.ospath('A/B/zeta')
+  epsilon_path = sbox.ospath('A/D/G/epsilon')
   files = [delta_path, zeta_path, epsilon_path]
 
-  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
+  exit_code, output, err = svntest.actions.run_and_verify_svn(None, [],
                                                               'revert',
                                                               '--recursive',
                                                               wc_dir)
@@ -291,12 +293,12 @@ def revert_add_directories(sbox):
   wc_dir = sbox.wc_dir
 
   # Revert our changes recursively from wc_dir.
-  X_path = os.path.join(wc_dir, 'X')
-  Y_path = os.path.join(wc_dir, 'A', 'C', 'Y')
-  Z_path = os.path.join(wc_dir, 'A', 'D', 'H', 'Z')
+  X_path = sbox.ospath('X')
+  Y_path = sbox.ospath('A/C/Y')
+  Z_path = sbox.ospath('A/D/H/Z')
   files = [X_path, Y_path, Z_path]
 
-  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
+  exit_code, output, err = svntest.actions.run_and_verify_svn(None, [],
                                                               'revert',
                                                               '--recursive',
                                                               wc_dir)
@@ -311,33 +313,39 @@ def revert_nested_adds(sbox):
   wc_dir = sbox.wc_dir
 
   # Revert our changes recursively from wc_dir.
-  X_path = os.path.join(wc_dir, 'X')
-  Y_path = os.path.join(wc_dir, 'A', 'C', 'Y')
-  Z_path = os.path.join(wc_dir, 'A', 'D', 'H', 'Z')
-  files = [X_path, Y_path, Z_path]
+  X_path = sbox.ospath('X')
+  Y_path = sbox.ospath('A/C/Y')
+  Z_path = sbox.ospath('A/D/H/Z')
+  files = ([X_path, Y_path, Z_path]
+           + [os.path.join(X_path, child)
+              for child in ['P', 'delta']]
+           + [os.path.join(Y_path, child)
+              for child in ['Q', 'epsilon', 'upsilon']]
+           + [os.path.join(Z_path, child)
+              for child in ['R', 'zeta']])
 
-  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
+  exit_code, output, err = svntest.actions.run_and_verify_svn(None, [],
                                                               'revert',
                                                               '--recursive',
                                                               wc_dir)
   check_reversion(files, output)
 
 #----------------------------------------------------------------------
-
+@SkipUnless(svntest.main.is_posix_os)
 def revert_add_executable(sbox):
   "revert: add some executable files"
 
   add_executable(sbox)
   wc_dir = sbox.wc_dir
 
-  all_path = os.path.join(wc_dir, 'all_exe')
-  none_path = os.path.join(wc_dir, 'none_exe')
-  user_path = os.path.join(wc_dir, 'user_exe')
-  group_path = os.path.join(wc_dir, 'group_exe')
-  other_path = os.path.join(wc_dir, 'other_exe')
+  all_path = sbox.ospath('all_exe')
+  none_path = sbox.ospath('none_exe')
+  user_path = sbox.ospath('user_exe')
+  group_path = sbox.ospath('group_exe')
+  other_path = sbox.ospath('other_exe')
   files = [all_path, none_path, user_path, group_path, other_path]
 
-  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
+  exit_code, output, err = svntest.actions.run_and_verify_svn(None, [],
                                                               'revert',
                                                               '--recursive',
                                                               wc_dir)
@@ -352,13 +360,13 @@ def revert_delete_files(sbox):
   wc_dir = sbox.wc_dir
 
   # Revert our changes recursively from wc_dir.
-  iota_path = os.path.join(wc_dir, 'iota')
-  mu_path = os.path.join(wc_dir, 'A', 'mu')
-  rho_path = os.path.join(wc_dir, 'A', 'D', 'G', 'rho')
-  omega_path = os.path.join(wc_dir, 'A', 'D', 'H', 'omega')
+  iota_path = sbox.ospath('iota')
+  mu_path = sbox.ospath('A/mu')
+  rho_path = sbox.ospath('A/D/G/rho')
+  omega_path = sbox.ospath('A/D/H/omega')
   files = [iota_path, mu_path, omega_path, rho_path]
 
-  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
+  exit_code, output, err = svntest.actions.run_and_verify_svn(None, [],
                                                               'revert',
                                                               '--recursive',
                                                               wc_dir)
@@ -373,18 +381,18 @@ def revert_delete_dirs(sbox):
   wc_dir = sbox.wc_dir
 
   # Revert our changes recursively from wc_dir.
-  E_path = os.path.join(wc_dir, 'A', 'B', 'E')
-  F_path = os.path.join(wc_dir, 'A', 'B', 'F')
-  H_path = os.path.join(wc_dir, 'A', 'D', 'H')
-  alpha_path = os.path.join(E_path, 'alpha')
-  beta_path  = os.path.join(E_path, 'beta')
-  chi_path   = os.path.join(H_path, 'chi')
-  omega_path = os.path.join(H_path, 'omega')
-  psi_path   = os.path.join(H_path, 'psi')
+  E_path = sbox.ospath('A/B/E')
+  F_path = sbox.ospath('A/B/F')
+  H_path = sbox.ospath('A/D/H')
+  alpha_path = sbox.ospath('A/B/E/alpha')
+  beta_path  = sbox.ospath('A/B/E/beta')
+  chi_path   = sbox.ospath('A/D/H/chi')
+  omega_path = sbox.ospath('A/D/H/omega')
+  psi_path   = sbox.ospath('A/D/H/psi')
   files = [E_path, F_path, H_path,
            alpha_path, beta_path, chi_path, omega_path, psi_path]
 
-  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
+  exit_code, output, err = svntest.actions.run_and_verify_svn(None, [],
                                                               'revert',
                                                               '--recursive',
                                                               wc_dir)
@@ -404,7 +412,7 @@ def revert_delete_dirs(sbox):
 # revert' or 'svn rm' will make that happen by removing the entry from
 # .svn/entries file. While 'svn revert' does with no error,
 # 'svn rm' does it with error.
-
+@Issue(863)
 def unschedule_missing_added(sbox):
   "unschedule addition on missing items"
 
@@ -412,15 +420,15 @@ def unschedule_missing_added(sbox):
   wc_dir = sbox.wc_dir
 
   # Create some files and dirs, then schedule them for addition
-  file1_path = os.path.join(wc_dir, 'file1')
-  file2_path = os.path.join(wc_dir, 'file2')
-  dir1_path = os.path.join(wc_dir, 'dir1')
-  dir2_path = os.path.join(wc_dir, 'dir2')
+  file1_path = sbox.ospath('file1')
+  file2_path = sbox.ospath('file2')
+  dir1_path = sbox.ospath('dir1')
+  dir2_path = sbox.ospath('dir2')
 
   svntest.main.file_append(file1_path, "This is the file 'file1'.")
   svntest.main.file_append(file2_path, "This is the file 'file2'.")
-  svntest.main.run_svn(None, 'add', file1_path, file2_path)
-  svntest.main.run_svn(None, 'mkdir', dir1_path, dir2_path)
+  sbox.simple_add('file1', 'file2')
+  sbox.simple_mkdir('dir1', 'dir2')
 
   # Make sure the 4 adds show up as such in status
   expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
@@ -440,9 +448,13 @@ def unschedule_missing_added(sbox):
   svntest.main.safe_rmtree(dir2_path)
 
   # Unschedule the additions, using 'svn rm' and 'svn revert'.
+  # FILE1_PATH will throw an error. DIR1_PATH will not since the stub is
+  # still available in the parent directory.
   svntest.main.run_svn(svntest.verify.AnyOutput, 'rm', file1_path)
-  svntest.main.run_svn(svntest.verify.AnyOutput, 'rm', dir1_path)
-  svntest.main.run_svn(None, 'revert', file2_path, dir2_path)
+  ### actually, the stub does not provide enough information to revert
+  ### the addition, so this command will fail. marking as XFail
+  sbox.simple_rm('dir1')
+  sbox.simple_revert('file2', 'dir2')
 
   # 'svn st' should now show absolutely zero local mods.
   expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
@@ -453,22 +465,22 @@ def unschedule_missing_added(sbox):
 #
 # Make sure 'rm foo; svn rm foo' works on files and directories.
 # Also make sure that the deletion is committable.
-
+@Issue(962)
 def delete_missing(sbox):
   "schedule and commit deletion on missing items"
 
   sbox.build()
   wc_dir = sbox.wc_dir
 
-  mu_path = os.path.join(wc_dir, 'A', 'mu')
-  H_path = os.path.join(wc_dir, 'A', 'D', 'H')
+  mu_path = sbox.ospath('A/mu')
+  H_path = sbox.ospath('A/D/H')
 
   # Manually remove a file and a directory.
   os.remove(mu_path)
   svntest.main.safe_rmtree(H_path)
 
   # Now schedule them for deletion anyway, and make sure no error is output.
-  svntest.actions.run_and_verify_svn(None, None, [], 'rm', mu_path, H_path)
+  sbox.simple_rm('A/mu', 'A/D/H')
 
   # Commit the deletions.
   expected_output = svntest.wc.State(wc_dir, {
@@ -483,35 +495,31 @@ def delete_missing(sbox):
 
   svntest.actions.run_and_verify_commit(wc_dir,
                                         expected_output,
-                                        expected_status,
-                                        None, wc_dir)
+                                        expected_status)
 
 #----------------------------------------------------------------------
 # Regression test for issue #854:
 # Revert . inside an svn added empty directory should generate an error.
-
+# Not anymore!  wc-ng uses absolute paths for everything, which means we
+# can handle this case without too much trouble.
+@Issue(854)
 def revert_inside_newly_added_dir(sbox):
   "revert inside a newly added dir"
 
   sbox.build(read_only = True)
-  wc_dir = sbox.wc_dir
-
-  os.chdir(wc_dir)
 
   # Schedule a new directory for addition
-  os.mkdir('foo')
-  svntest.main.run_svn(None, 'add', 'foo')
+  sbox.simple_mkdir('foo')
 
   # Now change into the newly added directory, revert and make sure
-  # an error is output.
-  os.chdir('foo')
-  svntest.actions.run_and_verify_svn(None, None, svntest.verify.AnyOutput,
-                                     'revert', '.')
+  # no error is output.
+  os.chdir(sbox.ospath('foo'))
+  svntest.main.run_svn(None, 'revert', '.')
 
 #----------------------------------------------------------------------
 # Regression test for issue #1609:
 # 'svn status' should show a schedule-add directory as 'A' not '?'
-
+@Issue(1609)
 def status_add_deleted_directory(sbox):
   "status after add of deleted directory"
 
@@ -528,12 +536,13 @@ def status_add_deleted_directory(sbox):
   # svn ci wc -m r2
   # svn mkdir wc/foo
 
-  A_path = os.path.join(wc_dir, 'A')
-  svntest.actions.run_and_verify_svn(None, None, [], 'rm', A_path)
+  A_path = sbox.ospath('A')
+
+  sbox.simple_rm('A')
   svntest.main.safe_rmtree(A_path)
-  svntest.actions.run_and_verify_svn(None, None, [],
-                                     'ci', '-m', 'log msg', wc_dir)
-  svntest.actions.run_and_verify_svn(None, None, [], 'mkdir', A_path)
+  sbox.simple_commit()
+
+  sbox.simple_mkdir('A')
 
   expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
   expected_status = svntest.wc.State(wc_dir,
@@ -545,7 +554,7 @@ def status_add_deleted_directory(sbox):
 
   # Update will *not* remove the entry for A despite it being marked
   # deleted.
-  svntest.actions.run_and_verify_svn(None, ['At revision 2.\n'], [],
+  svntest.actions.run_and_verify_svn(exp_noop_up_out(2), [],
                                      'up', wc_dir)
   expected_status.tweak('', 'iota', wc_rev=2)
   svntest.actions.run_and_verify_status(wc_dir, expected_status)
@@ -554,18 +563,18 @@ def status_add_deleted_directory(sbox):
 #----------------------------------------------------------------------
 # Regression test for issue #939:
 # Recursive 'svn add' should still traverse already-versioned dirs.
+@Issue(939)
+@Issue(4241)
 def add_recursive_already_versioned(sbox):
   "'svn add' should traverse already-versioned dirs"
 
+  sbox.build()
   wc_dir = sbox.wc_dir
 
-  if svntest.actions.make_repo_and_wc(sbox):
-    return 1
-
   # Create some files, then schedule them for addition
-  delta_path = os.path.join(wc_dir, 'delta')
-  zeta_path = os.path.join(wc_dir, 'A', 'B', 'zeta')
-  epsilon_path = os.path.join(wc_dir, 'A', 'D', 'G', 'epsilon')
+  delta_path = sbox.ospath('delta')
+  zeta_path = sbox.ospath('A/B/zeta')
+  epsilon_path = sbox.ospath('A/D/G/epsilon')
 
   svntest.main.file_append(delta_path, "This is the file 'delta'.")
   svntest.main.file_append(zeta_path, "This is the file 'zeta'.")
@@ -584,8 +593,8 @@ def add_recursive_already_versioned(sbox):
   ### or else Subversion will think you're trying to add the working copy
   ### to its parent directory, and will (possibly, if the parent directory
   ### isn't versioned) fail.
-  #svntest.main.run_svn(None, 'add', '--force', wc_dir)
-  #svntest.actions.run_and_verify_status(wc_dir, expected_status)
+  svntest.main.run_svn(None, 'add', '--force', wc_dir)
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
   # Now revert, and do the adds again from inside the working copy.
   svntest.main.run_svn(None, 'revert', '--recursive', wc_dir)
@@ -607,8 +616,7 @@ def fail_add_directory(sbox):
   os.makedirs(sbox.wc_dir)
 
   os.chdir(sbox.wc_dir)
-  svntest.actions.run_and_verify_svn('Failed mkdir',
-                                     None, svntest.verify.AnyOutput,
+  svntest.actions.run_and_verify_svn(None, svntest.verify.AnyOutput,
                                      'mkdir', 'A')
   if os.path.exists('A'):
     raise svntest.Failure('svn mkdir created an unversioned directory')
@@ -627,9 +635,94 @@ def delete_non_existent(sbox):
   wc_dir = sbox.wc_dir
 
   os.chdir(wc_dir)
-  svntest.actions.run_and_verify_svn(None, None, svntest.verify.AnyOutput,
+  svntest.actions.run_and_verify_svn(None, svntest.verify.AnyOutput,
                                      'rm', '--force', 'non-existent')
 
+
+#----------------------------------------------------------------------
+# Problem encountered by cmpilato when he inadvertantly upset an
+# 'svn rm --keep-local' and had to retry it.
+def delete_redelete_fudgery(sbox):
+  "retry of manually upset --keep-local deletion"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+  B_path = os.path.join(wc_dir, 'A', 'B')
+
+  # Delete 'A/B' using --keep-local, then remove at the OS level.
+  svntest.actions.run_and_verify_svn(None, [],
+                                     'rm', '--keep-local', B_path)
+  svntest.main.safe_rmtree(B_path)
+
+  # Update the tree.
+  #
+  ### When WC-NG is running in single-DB mode (one .svn directory and
+  ### database for the whole working copy), I suspect that this update
+  ### will change.  Today it re-adds the directory which we just
+  ### scheduled for deletion because the only record of that
+  ### scheduling is stored -- you guessed it -- the directory's .svn/
+  ### area... which we just deleted from disk.
+  ###
+  ### In single-DB-WC-NG-land, though, deleting the directory from
+  ### disk should have no bearing whatsoever on the scheduling
+  ### information stored now in the working copy root's one DB.  That
+  ### could change the whole flow of this test, possible leading us to
+  ### remove it as altogether irrelevant.  --cmpilato
+  svntest.actions.run_and_verify_svn(None, [], 'up', wc_dir)
+
+  # Now try to run
+  svntest.actions.run_and_verify_svn(None, [],
+                                     'rm', '--keep-local', B_path)
+
+def propset_on_deleted_should_fail(sbox):
+  "calling svn propset on a deleted node should fail"
+  sbox.build()
+  wc_dir = sbox.wc_dir
+  iota = os.path.join(wc_dir, 'iota')
+
+  svntest.actions.run_and_verify_svn(None, [], 'rm', iota)
+
+  svntest.actions.run_and_verify_svn(None, "svn: E155023: Can't set propert.*",
+                                     'ps', 'prop', 'val', iota)
+
+@Issue(3468)
+def replace_dir_delete_child(sbox):
+  "replace a dir, then delete a child"
+  # The purpose of this test is to make sure that when a child of a
+  # replaced directory is deleted, the result can be committed.
+
+  sbox.build()
+
+  # Replace A/D/H with a copy of A/B
+  sbox.simple_rm('A/D/H')
+  sbox.simple_copy('A/B', 'A/D/H')
+
+  # Remove two children
+  sbox.simple_rm('A/D/H/lambda')
+  sbox.simple_rm('A/D/H/E')
+
+  # Don't look at what "svn status" says before commit.  It's not clear
+  # what it should be and that's not the point of this test.
+
+  # Commit.
+  expected_output = svntest.wc.State(sbox.wc_dir, {
+    'A/D/H'         : Item(verb='Replacing'),
+    'A/D/H/lambda'  : Item(verb='Deleting'),
+    'A/D/H/E'       : Item(verb='Deleting'),
+    })
+
+  expected_status = svntest.actions.get_virginal_state(sbox.wc_dir, 1)
+  expected_status.add({
+    'A/D/H/F' : Item(status='  ', wc_rev=0),
+    })
+  expected_status.tweak('A/D/H', 'A/D/H/F', wc_rev=2)
+  expected_status.remove('A/D/H/psi', 'A/D/H/omega', 'A/D/H/chi')
+
+  svntest.actions.run_and_verify_commit(sbox.wc_dir,
+                                        expected_output,
+                                        expected_status)
+
+
 ########################################################################
 # Run the tests
 
@@ -639,7 +732,7 @@ test_list = [ None,
               revert_add_files,
               revert_add_directories,
               revert_nested_adds,
-              SkipUnless(revert_add_executable, svntest.main.is_posix_os),
+              revert_add_executable,
               revert_delete_files,
               revert_delete_dirs,
               unschedule_missing_added,
@@ -649,6 +742,9 @@ test_list = [ None,
               add_recursive_already_versioned,
               fail_add_directory,
               delete_non_existent,
+              delete_redelete_fudgery,
+              propset_on_deleted_should_fail,
+              replace_dir_delete_child,
              ]
 
 if __name__ == '__main__':
